/*
 * Copyright (C) 2005,2007 Pascal Terjan <pterjan@mandriva.com>
 *     
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ruby.h"
#include "libldetect.h"

VALUE cDetect;
VALUE cDetectEntry;

#define Struct2Hash(name,conv) \
	if(entry->name) { rb_hash_aset(rbentry, rb_str_new2(#name), conv(entry->name)); }

static VALUE new_detect_entry(struct pciusb_entry * entry) {
    VALUE rbentry;
    if(!entry) return Qnil;
    rbentry = rb_obj_alloc(cDetectEntry);
    Struct2Hash(vendor,INT2NUM)
    Struct2Hash(device,INT2NUM)
    Struct2Hash(subvendor,INT2NUM)
    Struct2Hash(subdevice,INT2NUM)
    Struct2Hash(class_id,INT2NUM)
    Struct2Hash(class,rb_str_new2)
    Struct2Hash(pci_bus,INT2NUM)
    Struct2Hash(pci_device,INT2NUM)
    Struct2Hash(pci_function,INT2NUM)
    Struct2Hash(pci_domain,INT2NUM)
    Struct2Hash(module,rb_str_new2)
    Struct2Hash(text,rb_str_new2)
    Struct2Hash(already_found,INT2NUM)
    return rbentry;
}    

enum probe_type {PCI, USB};

static VALUE ldetect_probe(VALUE self, enum probe_type pt) {
    struct pciusb_entries entries;
    VALUE rbentries;
    int i;
    entries = (pt == PCI)?pci_probe():usb_probe();
    rbentries = rb_ary_new();
    for(i=0; i<entries.nb; i++) {
	rb_ary_push(rbentries,new_detect_entry(&(entries.entries[i])));
    }
    pciusb_free(&entries);
    return rbentries;
}

static VALUE ldetect_pci_probe(VALUE self) {
    return ldetect_probe(self, PCI);
}

static VALUE ldetect_usb_probe(VALUE self) {
    return ldetect_probe(self, USB);
}	

static VALUE ldetect_pci_class2text(VALUE self, VALUE class) {
    return rb_str_new2(pci_class2text(NUM2INT(class)));
}

static VALUE ldetect_usb_class2text(VALUE self, VALUE class) {
    VALUE text, text2, text3;
    VALUE pipe = rb_str_new2("|");
    struct usb_class_text usb_text;
    usb_text = usb_class2text(NUM2INT(class));
    text = rb_str_new2(usb_text.usb_class_text);
    text2 = usb_text.usb_sub_text?rb_str_new2(usb_text.usb_sub_text):Qnil;
    text3 = usb_text.usb_prot_text?rb_str_new2(usb_text.usb_prot_text):Qnil;
    if (text2!=Qnil) {
	rb_str_append(text, pipe);
	rb_str_append(text, text2);
    }
    if (text3!=Qnil) {
        rb_str_append(text, pipe);
        rb_str_append(text, text3);
    }
    return text;
}

void Init_ldetect() {
    cDetect = rb_define_module("LDetect");
    rb_define_singleton_method(cDetect, "pci_probe", ldetect_pci_probe, 0);
    rb_define_singleton_method(cDetect, "usb_probe", ldetect_usb_probe, 0);
    rb_define_singleton_method(cDetect, "pci_class2text", ldetect_pci_class2text, 1);
    rb_define_singleton_method(cDetect, "usb_class2text", ldetect_usb_class2text, 1);

    cDetectEntry = rb_define_class("LDetectEntry", rb_cHash);
}
